;
; ASMTEST.ASM - Sample assembly language library for ACUCOBOL-85
;
; Date-Written: 20-Apr-89 - T. Drake Coker
;

;
; Start the library with the required routine-name table
;

LIB_TABLE	SEGMENT BYTE PUBLIC
		ASSUME CS:LIB_TABLE

LIB_START:
	DB	'ASMLIB'		;Start of every library

L1LEN	DB	L1ADDR - L1LEN - 1	;Directory entry for DOS-VERSION
	DB	'DOS-VERSION'
L1ADDR	DD	DOS_VERSION

L2LEN	DB	L2ADDR - L2LEN - 1	;Entry for GET-CURRENT-DIRECTORY
	DB	'GET-CURRENT-DIRECTORY'
L2ADDR	DD	GET_CURRENT_DIRECTORY

	DB	00			;end of directory

LIB_TABLE	ENDS


BINARY		EQU	11	;type # for signed binary values
PARAM_ERR	EQU	99	;return value to indicate parameter error

; 
; Layout structure to hold passed paramters
;

PASSED_PARAMS		STRUC

NUM_PARAMS		DW	1 DUP (?)
PARAM_1_ADDR		DD	1 DUP (?)
PARAM_1_SIZE		DW	2 DUP (?)
PARAM_1_TYPE		DB	2 DUP (?)
PARAM_1_DIGITS		DB	1 DUP (?)
PARAM_1_SCALE		DB	1 DUP (?)
PARAM_1_FILLER		DB	4 DUP (?)	;rest is unused

PARAM_2_ADDR		DD	1 DUP (?)
PARAM_2_SIZE		DW	2 DUP (?)
PARAM_2_TYPE		DB	2 DUP (?)
PARAM_2_DIGITS		DB	1 DUP (?)
PARAM_2_SCALE		DB	1 DUP (?)
PARAM_2_FILLER		DB	4 DUP (?)	;rest is unused

PASSED_PARAMS		ENDS


CODE_SEG	SEGMENT BYTE PUBLIC
		ASSUME CS:CODE_SEG

;
; DOS-VERSION - return MS-DOS major and minor version numbers in
; two PIC 9(4) BINARY parameters.
;

DOS_VERSION	PROC FAR

	CALL	CONVERT_PARAMS	;correct parameter addresses
;
; Check for legal parameters 
;
	CMP	ES:[BX].NUM_PARAMS,2		;check for 2 parameters
	JNE	DOS_ERR
	CMP	ES:[BX].PARAM_1_TYPE,BINARY	;insure parameter is BINARY
	JNE	DOS_ERR
	CMP	ES:[BX].PARAM_1_SIZE,2	 	;insure 2 bytes
	JNE	DOS_ERR
	CMP	ES:[BX].PARAM_2_TYPE,BINARY	;same for minor version #
	JNE	DOS_ERR
	CMP	ES:[BX].PARAM_2_SIZE,2
	JNE	DOS_ERR
;
	PUSH	BX				;save param pointer
	MOV	AH,30H				;call DOS to get version #
	INT	21H
	POP	BX
	MOV	DL,AH				;set DX to minor version #
	XOR	DH,DH
	XOR	AH,AH				;set AX to major version #
;
; Now save the values.  Note that BINARY formatted numbers are
; stored in the reverse byte-order from that used by the 80x86 processor.
; Thus we must XCHG the bytes of each word before we store them.
;
	LDS	DI,ES:[BX].PARAM_1_ADDR
	XCHG	AH,AL  	 
	MOV	[DI],AX	
	LDS	DI,ES:[BX].PARAM_2_ADDR
	XCHG	DH,DL
	MOV	[DI],DX
;
	MOV	AX,0				;indicate success
	RET
DOS_ERR:
	MOV	AX,PARAM_ERR
	RET

DOS_VERSION	ENDP

;
; GET_CURRENT_DIRECTORY - return current directory followed by LOW-VALUES
;

GET_CURRENT_DIRECTORY	PROC FAR

	CALL	CONVERT_PARAMS
	CMP	ES:[BX].NUM_PARAMS,1		;insure 1 parameter
	JNE	GET_ERR
	CMP	ES:[BX].PARAM_1_SIZE,64		;insure at least 64 bytes long
	JL	GET_ERR
;
	MOV	AH,47H				;call DOS to get directory
	SUB	DX,DX
	LDS	SI,ES:[BX].PARAM_1_ADDR
	INT	21H
;
	SUB	AX,AX				;return okay
	RET
GET_ERR:
	MOV	AX,PARAM_ERR			;indicate error
	RET

GET_CURRENT_DIRECTORY	ENDP

;
; CONVERT_PARAMS - This should be called by every library routine.
; It takes the parameter list (pointed to by ES:BX) and converts
; the data addresses to SEGMENT:OFFSET format.  This routine does
; not modify any registers.  
;

CONVERT_PARAMS	PROC NEAR

	PUSH	AX			;save registers used
	PUSH	BX
	PUSH	CX
	PUSH	DX
	MOV	CX,ES:[BX]		;CX = # of parameters
	ADD	BX,2
	JCXZ	CVT_END			;if no parameters, then all done
CVT_LP:	MOV	DX,ES:[BX+2]		;set DX:AX to address
	MOV	AX,ES:[BX]
	MOV	DH,DL			;do conversion
	MOV	DL,AH
	XOR	AH,AH
	SHL	DX,1
	SHL	DX,1
	SHL	DX,1
	SHL	DX,1
	MOV	ES:[BX+2],DX		;save converted address
	MOV	ES:[BX],AX
	ADD	BX,16			;point to next parameter
	LOOP	CVT_LP
CVT_END:
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	RET

CONVERT_PARAMS	ENDP

CODE_SEG	ENDS

	END	LIB_START

