PROGRAM Artf;

USES
  Crt,
  Graph,
  Forms;

CONST
   Memory  = 100;
   Windows =   4;

TYPE
  ResolutionPreference = (Lower, Higher);

  MyFracTyp = Record
                RInf,
                RSup,
                IInf,
                ISup,
                R0,
                I0        : Real;
                Iteration : LongInt;
                TypeGraph : String [5];
              End; { MyFracTyp }

VAR

    MaxColors,
    BackColor,
    MaxDelta,
    GraphMode,
    GraphDriver,
    Xmax, Ymax,
    VarR, VarI,
    itmax,
    compt,
    r ,
    i,
    k                : integer;

    pasR, pasI,
    xr,   xi,
    xr1,  xi1,
    mm,   nn,
    yr,   yi,
    cr,   ci,
    s                : Real;

    C                : Char;
    MyFractal        : MyFracTyp;
    Palette,
    OldPalette       : PaletteType;

Procedure Initialiser;

Var

  F: Form;

Begin

  TextBackGround (Black);
  TextColor (Yellow);
  ClrScr;
  Color (ForeColor);
  GotoXY(1, 1); ClrEol;
  Write(' F R A C T A L S : Saisie des valeurs initiales');
  GotoXY(1, 25); ClrEol;
  Write(' F2-Dessine   Esc-Quitte');

  F.Init(16, 8, 59, 17);
  F.Add (New (FRealPtr,
             Init(3, 2, ' Bornes rles,      Infrieure : ', 7, 4)));
  F.Add (New (FRealPtr,
             Init(3, 3, '                     Suprieure : ', 7, 4)));
  F.Add (New (FRealPtr,
             Init(3, 4, ' Bornes imaginaires, Infrieure : ', 7, 4)));
  F.Add (New (FRealPtr,
             Init(3, 5, '                     Suprieure : ', 7, 4)));
  F.Add (New (FRealPtr,
             Init(3, 6, ' Valeurs initiales,  Rle      : ', 7, 4)));
  F.Add (New (FRealPtr,
             Init(3, 7, '                     Imaginaire : ', 7, 4)));
  F.Add (New (FIntPtr,
             Init(3, 8, ' Maximum d''itrations           : ', 0, 999)));
  F.Add (New (FStrPtr,
             Init(3, 9, ' Type d''ensemble, (JULIA, MANDE): ',5)));

  F.Put (MyFractal);
  F.Show (True);
  If F.Edit <> CEsc
     Then F.Get (MyFractal)
     Else Halt (0);
  F.Done;
  NormVideo;

  ClrScr;
  ItMax := MyFractal.Iteration;
  xr    := MyFractal.R0;
  xi    := MyFractal.I0;

end;

Procedure fractals;
begin
    for r := 0 to VarR do
    begin
        cr := MyFractal.RInf + r * pasR;
        for i := 0 to VarI do
        begin
            ci := MyFractal.IInf + i * pasI;
            xr1 := xr;
            xi1 := xi;
            compt := 0;
            repeat
                 compt := compt + 1;
                 mm := xr1 * xr1;
                 nn := xi1 * xi1;
                 yr := mm - nn + cr;
                 yi := (2 * xr1 * xi1) + ci;
                 xr1 := yr;
                 xi1 := yi;
                 s := mm + nn
            until ( s  >= 4 ) or ( compt = itmax );
            PutPixel (r, i, compt mod 16)
        end
    end
end;


PROCEDURE FullPort;
{ Dfinit l'cran complet comme fentre active (ViewPort) }
BEGIN
  SetViewPort(0, 0, Xmax, Ymax, ClipOn);
END; { De FullPort }


PROCEDURE TestGraphError(GraphErr: INTEGER);
BEGIN
  IF GraphErr <> grOk THEN
    BEGIN
      Writeln('Erreur graphique : ', GraphErrorMsg(GraphErr));
      REPEAT UNTIL KeyPressed;
      Halt(1);
    END;
END;

PROCEDURE Init;
VAR
  Err, I	: INTEGER;
  StartX, StartY: INTEGER;
  Resolution	: ResolutionPreference;
  s		: STRING;
BEGIN
  Resolution := Lower;
  IF ParamCount > 0 THEN
    BEGIN
      s := ParamStr(1);
      IF s[1] = '/' THEN
	IF Upcase(s[2]) = 'H' THEN
	  Resolution := Higher;
    END;

  GraphDriver := Detect;
  DetectGraph(GraphDriver, GraphMode);
  TestGraphError(GraphResult);
  CASE GraphDriver OF
    CGA        : BEGIN
		   MaxDelta    := 7;
		   GraphDriver := CGA;
		   GraphMode   := CGAC1;
		 END;

    MCGA       : BEGIN
		   MaxDelta := 7;
		   CASE GraphMode of
		     MCGAMed, MCGAHi: GraphMode := MCGAC1;
		   END;
		 END;

    EGA		: BEGIN
		    MaxDelta := 16;
		    IF Resolution = Lower THEN
		      GraphMode := EGALo
		    ELSE
		      GraphMode := EGAHi;
		  END;

    EGA64	: BEGIN
		    MaxDelta := 16;
		    IF Resolution = Lower THEN
		      GraphMode := EGA64Lo
		    ELSE
		      GraphMode := EGA64Hi;
		  END;

     HercMono	: MaxDelta := 16;
     EGAMono	: MaxDelta := 16;
     PC3270	: BEGIN
		    MaxDelta := 7;
		    GraphDriver := CGA;
		    GraphMode	:= CGAC1;
		  END;


     ATT400	: CASE GraphMode OF
		    ATT400C1,
		    ATT400C2,
		    ATT400Med,
		    ATT400Hi  :
		      BEGIN
			MaxDelta  := 7;
			GraphMode := ATT400C1;
		      END;
		  END;

     VGA	 : BEGIN
		     MaxDelta := 16;
		   END;
  END; { Du case GraphDriver }
  InitGraph(GraphDriver, GraphMode, '');
  TestGraphError(GraphResult);
  SetTextStyle(DefaultFont, HorizDir, 1);
  SetTextJustify(CenterText, TopText);
  GetPalette(Palette);
  OldPalette := Palette;

  MaxColors := GetMaxColor;
  BackColor := 0;
  VarR := GetMaxX;
  VarI := GetMaxY;
END; { De Procdure Init}

BEGIN
  With MyFractal Do
  Begin
    RInf      := -3;
    Rsup      :=  3;
    IInf      := -3;
    ISup      :=  3;
    R0        :=  0;
    I0        :=  0;
    Iteration := 15;
    TypeGraph := 'MANDEL';
  End;
  While True Do
  Begin
   Initialiser;
   Init;
   PasR := ( MyFractal.RSup - MyFractal.RInf) / VarR;
   PasI := ( MyFractal.ISup - MyFractal.IInf) / VarI;
   Fractals;
   Repeat
     SetPalette (Random (Palette.Size), Random (Palette.Size));
   Until KeyPressed;
   C := ReadKey;
   CloseGraph;
   RestoreCrtMode;
   SetAllPalette (OldPalette)
  End
END.

